/* Copyright © 2010 Brandon L Black <blblack@gmail.com>
 *
 * This file is part of gdnsd.
 *
 * gdnsd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gdnsd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

// DNSCurve-related data types.  They had to be moved out of
//  dnscurve.h because of mutual #include dependencies between
//  dnscurve.h and dnspacket.h.

#ifndef _GDNSD_DNSCURVE_DATA_H
#define _GDNSD_DNSCURVE_DATA_H

#include "gdnsd.h"
#include <inttypes.h>
#include <stdbool.h>

typedef struct {
    uint64_t nbuf[8];     // Nonce output buffer
    uint8_t k[32];        // Key
    uint8_t in[16];       // Internal Nonce+State
    unsigned nbuf_remain; // Nonce64's remaining in buffer
    uint64_t c;           // Counter
} nonce64_state_t;

typedef struct {
    uint8_t key[32];
    uint32_t sep_mask;
    uint32_t sep_val;
} io_curvekey_t;

typedef struct _kc_data_struct kc_data_t;

// kc_data_t is a single data member of the cache,
//  storing a public key and our cached shared keys
//  with this peer.
struct _kc_data_struct {
    kc_data_t* next;       // next kc_data_t in the hash collision chain
    kc_data_t** linked_at; // where my own address is stored
                           // for hash purposes (the table slot
                           // itself or someone else's "next")
    unsigned shared_init;  // how many elements of shared have been init?
    uint32_t clock_data;   // CLOCK book-keeping
    uint8_t pubkey[32];
    uint8_t shared[];      // 32 * pctx->num_curve_keys
};

// kc_t is a key cache, caching up to a fixed count
//  of kc_data_t's with some kind of semi-intelligent
//  cache eviction strategy.
typedef struct {
    uint32_t salt;       // salt for the lookup3 hash
    unsigned size;       // cache size as number of slots
    unsigned hand;       // the CLOCK hand
    unsigned data_size;  // size of kc_data_t for this kc_t in bytes
    satom_t* cache_hit;  // stat counter for cache hits
    satom_t* cache_miss; // stat counter for cache misses
    uint8_t* data_array; // actually an array of kc_data_t, but uint8_t*
                         //   makes the pointer math simpler
    kc_data_t** hash;    // (size*2)-slot hashtable into "array" (max load 0.5)
} kc_t;

#endif // _GDNSD_DNSCURVE_DATA_H
